/**
 * This is the auto repair shop simulator. An AutoRepairShop is described as
 * a collection of ServiceBays and a parking lot of fixed capacity.  In the
 * parkinglot there is a set of Vehicles waiting to be serviced.  At most one
 * Vehicle arrives in one time unit.  The simulation assigns the Vehicles
 * to the ServiceBays. 
 * 
 * The fields of this class represent the main objects involved in the
 * simulation of the car shop:
 * 1. the Vehicles in the queue (parking lot)
 * 2. the ServiceBays in the repair shop
 * 3. the original queue positions of the Vehicles currently being serviced
 * 4. the curent time of the simulation in time units. 
 * 
 * To simulate the time aspect of our problem we use a loop. Each iteration 
 * represents one time unit and at each iteration we try to assign Vehicles 
 * to any idle ServiceBays and work on the Vehicle in each ServiceBay.  If the
 * work on a Vehicle is finished, then we put that Vehicle back in the parking
 * lot and assign another Vehicle to the ServiceBay if possible; otherwise we
 * leave that ServiceBay idle.
 * 
 * Check for completed Tasks and assign Vehicles to the ServiceBays at the 
 * top of each iteration.
 * 
 */
public class AutoRepairShop {
  
  private Vehicle[] vehicles; // the vehicles in the queue (parking lot)
  
  private ServiceBay[] bays; // the work bays in the repair shop
  
  private int[] originalPositions; // originalPositions[i] is the original
                                   // queue position of the Vehicle in bays[i].
                                   // We keep track of the positions in order
                                   // to put the Vehicles back to the right
                                   // parking spots after servicing.
                               
  private int currentTime = 0; // the curent time of the simulation
  
  private int firstAvailableSpot; // first empty slot for simulations that
                              // dynamically insert cars as they arrive
  
  
  /** Constructor of the AutoRepairShop that has an empty queue of cars.
   * 
   * @param bays -
   *            The repair bays of the car shop.
   * @param parkingLotSpace -
   *            the maximum number of cars that can fit in the parking lot and
   *            can be serviced by the shop
   * 
   */
  public AutoRepairShop(ServiceBay[] bays, int parkingLotSpace) {
    this.bays = bays;
    this.originalPositions = new int[this.bays.length];
    for (int i = 0; i< originalPositions.length; i++) {
     originalPositions[i] = -1;  //signal 
    }
    this.vehicles = new Vehicle[parkingLotSpace];
    this.firstAvailableSpot = 0;
  }
  
  /**
   * This method inserts Vehicle v in the first available slot in the parking
   * lot if v is not null.  If we run out of slots it just prints the message 
   * "Lot is full--please come tomorrow" and does not try to insert v.
   * It uses the field firstAvailableSpot to keep track of the first avalable 
   * slot.
   * 
   * @param v -
   *            Vehicle to insert in the queue
   */
  public void insertVehicle(Vehicle v) {

    //---------- Implement this method ----------

    
  }
  
  /**
   * This method is called to attempt to put a Vehicle into the ServiceBay 
   * indicated by the parameter serviceBayIndex.  If a serviced Vehicle is 
   * currently in that bay, put the Vehicle back in the lot first.  Then use a
   * a simple greedy algorithm:  choose the first Vehicle, based on its 
   * position in the queue, that is waiting for service and can be serviced 
   * in that ServiceBay.  If no such Vehicle is found, the method makes the 
   * bay idle by assigning null to the bay's currentVehicle field.  
   * 
   * When assigning a Vehicle to the ServiceBay, empty the place in
   * the parking lot (make it null) but keep track of that position using the 
   * array originalPositions so that the Vehicle can be put back after 
   * servicing.
   * 
   * @param serviceBayIndex -
   *            the index of the bay for which we choose the vehicle
   * @return the parking lot spot in which there was the assigned car
   */
  public void schedule(int serviceBayIndex) {

    //---------- Implement this method ----------

    
  }
  
  /**
   * This method simulates the the work in one time unit for all the 
   * ServiceBays in this AutoRepairShop.  It loops through the bays and works 
   * on the Task for the Vehicle in each bay.  If a bay is empty or the Task 
   * is finished at the begining of the iteration, the method tries to 
   * schedule a Vehicle for that bay.  Increment the system time.
   */
  public void work() {

    //---------- Implement this method ----------

    
  }
  
  /*
   * =String representation of this AutoRepairShop
   */
  public String toString() {
    String out = "Auto Service Shop: Current time " + this.currentTime
      + "\n Service bays:\n";
    for (int i = 0; i < this.bays.length; i++) {
      out += (i + 1) + ": " + this.bays[i].toString() + "\n\n";
    }
    out += "Vehicles in the parking lot:\n";
    for (int i = 0; i < this.vehicles.length; i++) {
      if ((this.vehicles[i] != null)
            && (!this.vehicles[i].getJob().isInProgress())) {
        out += (i + 1) + ": " + this.vehicles[i].toString() + "\n\n";
      }
    }
    return out;
  }
  
  /**
   * This method simulates the next arrival to the parking lot in 1 time unit.
   * There is a 1-in-5 chance (probability of 0.2) that no vehicle arrives--the method
   * returns null in this case.  Otherwise, a random vehicle from the vector vehicles 
   * is selected and returned.  This method never selects the same vehicle twice.  
   * In order to implement this behaviour, you need to set to null the element that
   * you are choosing to return.  If all the vehicles have been selected already,
   * the method returns null. 
   * 
   * @param vehicles -
   *            the vector of vehicles from which we choose
   * @return the chosen vehicle
   */
  public static Vehicle nextArrival(Vehicle[] vehicles){
    
    //---------- Implement this method ----------

    
    return null;  // Remove or revise this stub (dummy return statement) 
  }
  
  /**
   * This method runs the simulation. It creates a new AutoRepairShop from
   * the parameters and then it calls the method work() timeSteps number of
   * times.  After each iteration it prints the AutoRepairShop object.
   * 
   * @param bays -
   *            The repair bays of the car shop.
   * @param vehicles -
   *            The queue of vehicles waiting service
   * @param parkingLotSpace -
   *            the maximum number of cars that can fit in the parking lot and
   *            can be serviced by the shop
   * @param timeSteps
   *            The number of time steps for the simulation.
   * 
   */
  public static void simulate(ServiceBay[] bays, Vehicle[] vehicles,
                              int parkingLotSpace, int timeSteps) {
    AutoRepairShop crs = new AutoRepairShop(bays, parkingLotSpace);
    System.out.println("Starting Simulation....");
    while (crs.currentTime < timeSteps) {
      crs.insertVehicle(nextArrival(vehicles));
      crs.work();
      System.out.println("\n****************************\n");
      System.out.println(crs);
      System.out.println("\n****************************\n");
    }
    System.out.println("Simulation ended....\n\n");
  }
  
  /**
   * Answer the question in the String.  Write your answer as part of the String
   * so that it will be printed as output when your program is executed.
   */
  public static void answerQ1() {
    String s= "Suppose the Vehicle class does NOT contain the method acceptableAtBay";
       s += "\nbut the subclasses do define methods acceptableAtBay.  Is any change";
       s += "\nrequired in the rest of the program?  If so, write ONE example of the";
       s += "\nnecessary change below.  Be sure to say which method and which class";
       s += "\nyour example refers to.\n";
         
    // -------------------- Answer the above question --------------------
    
       
    System.out.println(s);   
  }

  /**
   * Answer the question in the String.  Write your answer as part of the String
   * so that it will be printed as output when your program is executed.
   */
  public static void answerQ2() {
    String s= "Suppose we want to make the simulation more realistic by allowing";
       s += "\nmultiple Vehicles to arrive in each time unit and the number of";
       s += "\nvehicles that arrive at the same time is not fixed.  How might we";
       s += "\nhandle this change in the methods nextArrival and insertVehicle?";
       s += "\nAnswer in no more than 3 sentences and be specific about the ";
       s += "\nparameter and return types of those methods.\n";
         
    // -------------------- Answer the above question --------------------
         

    System.out.println(s);
  }

         
         
  /**
   * The main method is used to provide an example set of data for the
   * simulation.
   */
  public static void main(String[] args) {
    ServiceBay[] bays = new ServiceBay[4];
    int[] jt0 = { Task.DIAGNOSTIC, Task.MAINTENANCE, Task.STATE_INSPECTION };
    String[] vt0 = { "Car", "Truck" };
    bays[0] = new ServiceBay(jt0, vt0, 15, 20);
    int[] jt1 = { Task.DIAGNOSTIC, Task.REPAIR };
    String[] vt1 = { "Car", "Truck" };
    bays[1] = new ServiceBay(jt1, vt1, 15, 15);
    int[] jt2 = { Task.DIAGNOSTIC, Task.REPAIR };
    String[] vt2 = { "Car" };
    bays[2] = new ServiceBay(jt2, vt2, 12, 5);
    int[] jt3 = { Task.DIAGNOSTIC, Task.MAINTENANCE };
    String[] vt3 = { "Car" };
    bays[3] = new ServiceBay(jt3, vt3, 12, 5);
    
    Vehicle[] vehicles = {
      new Truck("VOLVO", "CD-56566", new Task(80, Task.REPAIR), 13, 10),
      new Car("Subaru", "CRD-1587", new Task(3, Task.MAINTENANCE), "4D Sedan"),
      new Car("Subaru", "CRV-4797", new Task(6, Task.REPAIR), "Outback"),
      new Truck("Ford", "DYU-9986", new Task(5, Task.DIAGNOSTIC), 9, 5),
      new Car("Honda", "DRB-6508", new Task(2, Task.MAINTENANCE), "4D Sedan"),
      new Car("Honda", "DRV-7408", new Task(2, Task.STATE_INSPECTION), "SUV"),
      new Truck("Dodge", "CFG-8089", new Task(10, Task.MAINTENANCE), 9, 6),
      new Car("Toyota", "SUV-1234", new Task(20, Task.REPAIR), "4D Sedan"),
      new Car("Toyota", "DHV-3997", new Task(1, Task.DIAGNOSTIC), "4D Sedan"),
      new Truck("Nisan", "HHG-7443", new Task(8, Task.STATE_INSPECTION), 9, 5),
      new Car("Volvo", "DEO-6543", new Task(10, Task.MAINTENANCE), "Convertible"),
      new Car("Nisan", "DFK-7443", new Task(7, Task.STATE_INSPECTION), "SUV") 
    };
    
    AutoRepairShop.simulate(bays, vehicles, 20, 15); //20 slots, 15 time units
    
    answerQ1();
    answerQ2();
  }
  
}
