/**
 * This class models a ServiceBay, i.e., a place in the garage that has
 * special equipment for working on Vehicles.  A ServiceBay has several 
 * properties:  1. the types of Tasks that it can perform;  2. the types of 
 * Vehicles that it can service;  3. the maximum weight and height of Trucks
 * that it can service;  4. the Vehicle that is being worked on in the bay.
 * 
 * The only changeble property is the current vehicle that is being workd on.
 * 
 */
class ServiceBay {
  private int[] allowableTasksTypes; // the types of tasks that can be
                                     // performed in the bay
  
  private String[] allowableVehicleTypes; // the types of vehicles that can be
                                          // serviced in the bay
  
  private int maxHeight; // the maximum height of a Truck that can be
                         // serviced in the bay.
  
  private int maxWeight; // the maximum weight of a Truck that can be
                         // serviced in the bay.
  
  private Vehicle currentVehicle; // the vehicle that is currently serviced in
                                  // the bay.  If null, the bay is idle.
  
  /**
   * This constructor initializes a ServiceBay to the specified 
   * characteristics.  No vehicle is initially assigned to the bay.
   * 
   * @param allowableTasksTypes
   *            Initializes the field allowableTasksTypes i.e. The types of
   *            tasks which can be performed by the bay
   * @param allowableVehicleTypes
   *            Initializes the field allowableVehicleTypes i.e. The types of
   *            tvehicles that can be serviced by the bay
   * @param height
   *            Initializes the field maxHeight i.e. the maximum height of a
   *            vehicle that can be serviced by the bay.
   * @param weight
   *            Initializes the field maxWeight i.e. the maximum weight of a
   *            vehicle that can be serviced by the bay.
   */
  public ServiceBay(int[] allowableTasksTypes,
                    String[] allowableVehicleTypes, int height, int weight) {
    this.allowableTasksTypes = allowableTasksTypes;
    this.allowableVehicleTypes = allowableVehicleTypes;
    maxHeight = height;
    maxWeight = weight;
    currentVehicle = null;
  }
  
  /**
   * This method is used to access the vehicle on which the bay is currently
   * working.
   * 
   * @return Returns field currentVehicle.
   */
  public Vehicle getCurrentVehicle() {
    return currentVehicle;
  }
  
  /**
   * This method is used to set/change the Vehicle that is currently being 
   * worked on in this ServiceBay.
   * 
   * @param currentVehicle
   *            The currentVehicle to set
   */
  public void setCurrentVehicle(Vehicle currentVehicle) {
    this.currentVehicle = currentVehicle;
  }
  
  /**
   * This method tests if a specific height value (of a Truck) can be 
   * accommodated in this ServiceBay.
   * 
   * @param height
   *            The height to be tested
   * @return returns true if the value is allowed and false otherwise.
   */
  public boolean allowHeight(int height) {
    return ((0 <= height) && (height <= this.maxHeight));
  }
  
  /**
   * This method tests if a specific weight value (of a Truck) can be 
   * accommodated in this ServiceBay.
   * 
   * @param weight
   *            The weight to be tested
   * @return returns true if the value is allowed and false otherwise.
   */
  public boolean allowWeight(int weight) {
    return 0<=weight && weight<=this.maxWeight;
  }
  
  /**
   * @param v
   *            The Vehicle to be tested
   * @return Returns true if v's type is allowed in this ServiceBay.
   *            Otherwise returns false.
   */
  public boolean allowVehicle(Vehicle v) {

    //---------- Implement this method ----------
    
    
    return false;  // Remove or revise this stub

  }
  
  /**
   * @param t
   *            The task to be tested
   * @return Returns true if t is a type of Task that can be performed in this
   *            ServiceBay.  Otherwise returns false.
   */
  public boolean allowTask(Task t) {

    //---------- Implement this method ----------
    
    
    return false;  // Remove or revise this stub
  }
  
  /*
   * =String representation of this ServiceBay.
   */
  public String toString() {
    String out = "Bay: \nAllowed vehicle types:";
    for (int i = 0; i < this.allowableVehicleTypes.length; i++)
      out += " " + this.allowableVehicleTypes[i];
    out += "\nAllowed tasks:";
    for (int i = 0; i < this.allowableTasksTypes.length; i++) {
      String taskType = "UNDEFINED";
      if (this.allowableTasksTypes[i] == Task.MAINTENANCE) {
        taskType = "MAINTENANCE";
      } else if (this.allowableTasksTypes[i] == Task.DIAGNOSTIC){
        taskType = "DIAGNOSTIC";
      } else if (this.allowableTasksTypes[i] == Task.REPAIR) {
        taskType = "REPAIR";
      } else if (this.allowableTasksTypes[i] ==  Task.STATE_INSPECTION){
        taskType = "STATE INSPECTION";
      }      
      
      out += " " + taskType;
    }
    String currentVehicleStr;
    if (this.currentVehicle == null) {
      currentVehicleStr = "Idle";
    } else {
      currentVehicleStr = this.currentVehicle.toString();
    }
    out += "\nMax height: " + this.maxHeight + " Max weight: " + this.maxWeight 
           + "\nWorking on: " + currentVehicleStr;
    return out;
  }
  
}
