/**
 * This class models a vehicle that can be serviced by a car repair shop. We use
 * this class to model the characteristics of any vehicle that comes to the shop
 * to be serviced. The main characteristics are the make of the vehicle and the 
 * license plate.
 * 
 * The vehicle has associated a Task to be performed by the service people. 
 * 
 */

class Vehicle {
  
  private String make; // make of the vehicle
  
  private String licensePlate; // license plate
  
  private Task job; // operation to be performed on the vehicle
  
  /** Constructor Initializes the all the fields of the object to the 
   * parameters supplied.
   * 
   * @param make
   *            make of the vehicle
   * @param licensePlate
   *            license plate
   * @param job
   *            operation to be prrformed on the vehicle
   */
  public Vehicle(String make, String licensePlate, Task job) {
    this.make = make;
    this.licensePlate = licensePlate;
    this.job = job;
  }
  
  /**
   * @return Returns the type of the vehicle. For the base class return "Vehicle"
   */
  public String getVehicleType() {
    return "Vehicle";
  }
    
  /** This method determines if a certain bay is able to service this Vehicle.
   * Since a ServiceBay is supposed to service specific types of Vehicles, for
   * this base class this method always returns false.  It should be 
   * overeridden in the child classes.
   * 
   * @param bay 
   *            Service bay to be considered for servicing the vehicle;
   * @return Returnes true if the bay is able to service the vehicle and
   *         false if it is not.
   */
  public boolean acceptableAtBay(ServiceBay bay) {
    return false;
  }
    
  /** This method is used to access the task associated with the vehicle
   * 
   * @return Returns field job i.e. the Task associated with the vehicle;
   */
  public Task getJob() {
    return job;
  }
  
  /*
   * String representation of the vehicle, see example for example output.
   */
  public String toString() {
    String s= this.getVehicleType()
      + "\n"
      + this.make
      + "  Licence Plate: "
      + this.licensePlate;
    if (this.getVehicleType().equals("Vehicle"))
      s += "\n" + this.job.toString();
    return s;
  }
  
}
